Texture2D<uint2> StencilBufferFlat3DTex;
matrix WorldViewProjection;

RasterizerState DisableCulling
{
  MultiSampleEnable = False;
  CullMode = None;
  ScissorEnable = true;
};

BlendState DisableBlending
{
  BlendEnable[0] = false;
  RenderTargetWriteMask[0] = 0x0F;
};

DepthStencilState InOutCounter
{
    DepthEnable = FALSE;
    DepthWriteMask = ZERO;
    StencilEnable = true;
    StencilReadMask = 0x00;
    StencilWriteMask = 0xFF;
    FrontFaceStencilFunc = Always;
    FrontFaceStencilPass = Decr;
    FrontFaceStencilFail = Keep;
    BackFaceStencilFunc = Always;
    BackFaceStencilPass = Incr;
    BackFaceStencilFail = Keep;
    
};

DepthStencilState DisableDepthStencil
{
    DepthEnable = FALSE;
    DepthWriteMask = ZERO;
    StencilEnable = FALSE;
    StencilReadMask = 0x00;
    StencilWriteMask = 0x00;
};

struct VSTransformIn
{
    float3 Pos          : POSITION;
};

struct VSTransformOut
{
    float4 Pos	        : SV_Position;
};

struct VSWriteBackIn
{
    float3 Pos          : POSITION;
    float3 Tex          : TEXCOORD;
};

struct VSWriteBackOut
{
    float4 Pos          : POSITION;
    float3 Tex          : TEXCOORD;
};

struct GSWriteBackOut
{
    float4 Pos          : SV_Position;
    float3 Tex          : TEXCOORD;
    uint   RTIndex      : SV_RenderTargetArrayIndex;
};

VSTransformOut VSTransform(VSTransformIn input)
{
    VSTransformOut output;
    output.Pos = mul(float4(input.Pos,1), WorldViewProjection);
    output.Pos.y *= -1;
    return output;
}

VSWriteBackOut VSWriteBack(VSWriteBackIn input)
{
    VSWriteBackOut output;
    output.Pos = float4(input.Pos,1);
    output.Tex = input.Tex;
    return output;
}

[maxvertexcount(3)]
void GSWriteBack(triangle VSWriteBackOut input[3], inout TriangleStream<GSWriteBackOut> triStream)
{
    GSWriteBackOut output;
    output.RTIndex = input[0].Tex.z;
    for(int v = 0; v < 3; v++)
    {
        output.Pos = input[v].Pos;
        output.Tex = input[v].Tex;
        triStream.Append(output);
    }
    triStream.RestartStrip();
}

float4 PSWriteBack(GSWriteBackOut input) : SV_Target
{
    if(StencilBufferFlat3DTex.Load(int3(input.Tex.x, input.Tex.y, 0)).g)
        return 1.0;
    return 0;
}

//----------------------------------------------------------------

technique10 Transform
{
    pass P0
    {
        SetVertexShader(CompileShader(vs_4_0, VSTransform()));
        SetGeometryShader(NULL);
        SetPixelShader(NULL);
        SetRasterizerState(DisableCulling);
        SetBlendState(DisableBlending, float4(0.0f, 0.0f, 0.0f, 0.0f), 0xFFFFFFFF);
        SetDepthStencilState(InOutCounter, 0);
    }
}

technique10 WriteBack
{
    pass P0
    {
        SetVertexShader(CompileShader(vs_4_0, VSWriteBack()));
        SetGeometryShader(CompileShader(gs_4_0, GSWriteBack()));
        SetPixelShader(CompileShader(ps_4_0, PSWriteBack()));
        SetRasterizerState(DisableCulling);
        SetBlendState(DisableBlending, float4(0.0f, 0.0f, 0.0f, 0.0f), 0xFFFFFFFF);
        SetDepthStencilState(DisableDepthStencil, 0);
    }
}