matrix World;
matrix View;
matrix Projection;
matrix WorldViewProjection;

float shCoeffs[16];

#include "SHLighting.fx"

RasterizerState DisableCulling
{
    CullMode = NONE;
};

//--------------------------------------------------------------------------------------
struct VS_INPUT
{
    float4 Pos : POSITION;
};

struct PS_INPUT
{
    float4 Pos : SV_POSITION;
    float3 SpherePos : TEXCOORD0;
};

SamplerState SamLinear
{
    Filter = MIN_MAG_MIP_LINEAR;
    AddressU = Clamp;
    AddressV = Clamp;
    AddressW = Clamp;
};

//--------------------------------------------------------------------------------------

PS_INPUT VSDebugSphere(VS_INPUT input)
{
    PS_INPUT output;
    output.SpherePos = input.Pos.xyz;
    output.Pos = mul(input.Pos, mul(mul(World, View),Projection));
    
    return output;
}

float4 PSDebugFunction(PS_INPUT input) : SV_Target
{
	float val = 0.0f;
	
	val += shCoeffs[0] * 1 / (2 * sqrt(PI));
	
	val += shCoeffs[1] * -input.SpherePos.z * sqrt(3) / (2 * sqrt(PI));
	val += shCoeffs[2] * input.SpherePos.y * sqrt(3) / (2 * sqrt(PI));
	val += shCoeffs[3] * -input.SpherePos.x * sqrt(3) / (2 * sqrt(PI));
	
	val += shCoeffs[4] * input.SpherePos.z * input.SpherePos.x * sqrt(15) / (2 * sqrt(PI));
	val += shCoeffs[5] * -input.SpherePos.y * input.SpherePos.z * sqrt(15) / (2 * sqrt(PI));
	val += shCoeffs[6] * (3 * input.SpherePos.y * input.SpherePos.y - 1) * sqrt(5) / (4 * sqrt(PI));
	val += shCoeffs[7] * -input.SpherePos.y * input.SpherePos.x * sqrt(15) / (2 * sqrt(PI));
	val += shCoeffs[8] * (input.SpherePos.x * input.SpherePos.x - input.SpherePos.z * input.SpherePos.z) * sqrt(15) / (4 * sqrt(PI));
	
	val += shCoeffs[9] * -(3 * input.SpherePos.x * input.SpherePos.x - input.SpherePos.z * input.SpherePos.z) * input.SpherePos.y * sqrt(35) * sqrt(2) / (8 * sqrt(PI));
	val += shCoeffs[10] * input.SpherePos.y * input.SpherePos.z * input.SpherePos.x * sqrt(105) / (2 * sqrt(PI));
	val += shCoeffs[11] * -(-1 + 5 * input.SpherePos.y * input.SpherePos.y) * input.SpherePos.z * sqrt(21) * sqrt(2) / (8 * sqrt(PI));
	val += shCoeffs[12] * input.SpherePos.y * (5 * input.SpherePos.y * input.SpherePos.y - 3) * sqrt(7) / (4 * sqrt(PI));
	val += shCoeffs[13] * -(-1 + 5 * input.SpherePos.y * input.SpherePos.y) * input.SpherePos.x * sqrt(21) * sqrt(2) / (8 * sqrt(PI));
	val += shCoeffs[14] * (input.SpherePos.x * input.SpherePos.x - input.SpherePos.z * input.SpherePos.z) * input.SpherePos.y * sqrt(105) / (4 * sqrt(PI));
	val += shCoeffs[15] * -(input.SpherePos.x * input.SpherePos.x - 3 * input.SpherePos.z * input.SpherePos.z) * input.SpherePos.x * sqrt(35) * sqrt(2) / (8 * sqrt(PI));
	
	return float4(val, -val, 0.0f, 1.0f);		// Not HDR Compatible
}

//--------------------------------------------------------------------------------------
technique10 RenderDebugFunction
{
    pass P0
    {
        SetVertexShader(CompileShader(vs_4_0, VSDebugSphere()));
        SetGeometryShader(NULL);
        SetPixelShader(CompileShader(ps_4_0, PSDebugFunction()));
    }
}
