Texture2DArray	ShadowTex;
float4			Splits;
float4x4		SplitMatrices[4];
float2			LightLinNearFar;
float3			LightPosition;				  // Light position of the shadow map light in world coords

float    g_VSMMinVariance = 0.000001;         // Minimum variance for VSM
float	 g_LBRAmount      = 0.28;
float2   g_ShadowBias     = float2(0.5, 0);
const int SplitPowLookup[8] = {0, 1, 1, 2, 2, 2, 2, 3};

// Full anisotropic trilinear filtering
SamplerState sampAnisotropic
{
    AddressU = Clamp;
    AddressV = Clamp;
    Filter = ANISOTROPIC;
    MaxAnisotropy = 16;
};

struct Depth_VSIn
{
    float4 Pos      : POSITION;     
};

struct Depth_PSIn
{
     float4 Pos     : SV_Position;
     float3 PosView : PosView;     
};

Depth_PSIn Depth_VS(Depth_VSIn Input)
{
    Depth_PSIn Output;
    
    Output.Pos = mul(Input.Pos, WorldViewProjection);
    Output.PosView  = mul(Input.Pos, mul(World, View));
    //Output.PosView  = mul(Input.Pos, World);
    
    return Output;
}

float2 ComputeMoments(float Depth)
{
    float2 Moments;
    Moments.x = Depth;
    Moments.y = Depth*Depth;

    float dx = ddx(Depth);
    float dy = ddy(Depth);
    float Delta = 0.25 * (dx*dx + dy*dy);
    Moments.y += Delta;

    return Moments;
}

float linstep(float min, float max, float v)
{
    return clamp((v - min) / (max - min), 0, 1);
}

// Rescale into [0, 1]
float RescaleDistToLight(float Distance)
{
    return linstep(LightLinNearFar.x, LightLinNearFar.y, Distance);
}

// Compute which split we're in
int GetSplit(float SliceDepth)
{
    int Split = dot(1, SliceDepth > Splits);
    
    // Trick from GPU Gems
    // Ensure that every fragment in the quad choses the same split so that derivatives
    // will be meaningful for proper texture filtering and LOD selection.
    // Hehe... best trick/hack ever :)
    int SplitPow = 1 << Split;
    int SplitX = abs(ddx(SplitPow));
    int SplitY = abs(ddy(SplitPow));
    int SplitXY = abs(ddx(SplitY));
    int SplitMax = max(SplitXY, max(SplitX, SplitY));
    //SplitMax = min(SplitMax, 8);
    Split = SplitMax > 0 ? SplitPowLookup[SplitMax-1] : Split;
    
    return Split;
}

// Computes Chebyshev's Inequality
// Returns an upper bound given the first two moments and mean
float ChebyshevUpperBound(float2 Moments, float Mean, float MinVariance)
{
    // One-tailed inequality valid if Mean > Moments.x
    // else return 1.0
    float p = (Mean <= Moments.x);
    
    // Compute variance
    float Variance = Moments.y - (Moments.x * Moments.x);
    
    //Variance = min( 1.0f, max( 0.0f, Variance + 0.00001f ) );
    Variance = max(Variance, MinVariance);
    // Compute probabilistic upper bound
    
    float d = Moments.x - Mean;
    float p_max = Variance / (Variance + d*d);
    
    return max(p, p_max);
}

float3 CalcShadowContribution(float3 PosInWorld, float2 LightTexCoord, int Split)
{
    float3 DirToLight = LightPosition - PosInWorld;
    float DistToLight = length(DirToLight);
	
	float RescaledDist = RescaleDistToLight(DistToLight);
	float2 Moments = ShadowTex.Sample(sampAnisotropic, float3(LightTexCoord, Split)) + g_ShadowBias; //Momenst + Bias
    float ShadowContrib = ChebyshevUpperBound(Moments, RescaledDist, g_VSMMinVariance);

    //Reduce light bleed
    ShadowContrib = linstep(g_LBRAmount, 1, ShadowContrib);

    
    return ShadowContrib;
}

float2 Depth_PS(centroid Depth_PSIn Input) : SV_Target
{
    float Depth = RescaleDistToLight(length(Input.PosView));
    //float Depth = length(Input.PosView);
    float2 Moments = ComputeMoments(Depth) - g_ShadowBias; //Moments - Bias
    return Moments;
}

technique10 Depth
{
    pass P0
    {
        SetVertexShader(CompileShader(vs_4_0, Depth_VS()));
        SetGeometryShader(NULL);
        SetPixelShader(CompileShader(ps_4_0, Depth_PS()));
        SetBlendState(NoAlpha, float4( 0.0f, 0.0f, 0.0f, 0.0f ), 0xFFFFFFFF);
    }
}