matrix World;
matrix View;
matrix Projection;
matrix WorldViewProjection;
float3 LightDir[3];
float3 LightColor[3];
float3 AmbientColor;
float3 DiffuseColor;
float3 SpecularColor;
float3 EmissiveColor;
float3 CameraPos;
float Shininess;
int NumLights;
float Opacity;

float3 ClipPlaneNormal;
float  ClipPlaneDistance;

Texture2D Texture;

RasterizerState DisableCulling
{
    CullMode = NONE;
};

RasterizerState WireFrame
{
	FillMode = WIREFRAME;
	CullMode = NONE;
};

BlendState NoAlpha
{
    BlendEnable[0] = FALSE;
};

//--------------------------------------------------------------------------------------
struct VS_INPUT
{
    float4 Pos : POSITION;
	float2 Texcoord : TEXCOORD0;
};

struct PS_INPUT
{
    float4 Pos : SV_POSITION;
	float2 Texcoord : TEXCOORD0;
	float Distance : SV_ClipDistance0;
};

SamplerState SamLinear
{
    Filter = MIN_MAG_MIP_LINEAR;
    AddressU = Wrap;
    AddressV = Wrap;
    AddressW = Wrap;
};

SamplerState SamLinearClamp
{
    Filter = MIN_MAG_MIP_LINEAR;
    AddressU = Clamp;
    AddressV = Clamp;
    AddressW = Clamp;
};

//--------------------------------------------------------------------------------------

PS_INPUT VS(VS_INPUT input)
{
    PS_INPUT output;
	
	//Camera position of plane is always center of skybox
	input.Pos.xyz = input.Pos.xyz + CameraPos.xyz;
	float4 PosInWorld = mul(input.Pos, World);
	
	output.Pos = mul(input.Pos, WorldViewProjection);
	output.Texcoord = input.Texcoord;
    
    output.Distance = ClipPlaneNormal.x * PosInWorld.x + ClipPlaneNormal.y *
    PosInWorld.y + ClipPlaneNormal.z * PosInWorld.z + ClipPlaneDistance;
    
    return output;
}

float3 PS(PS_INPUT input) : SV_Target
{
    return Texture.Sample(SamLinearClamp, input.Texcoord).rgb;
}


//--------------------------------------------------------------------------------------
technique10 Render
{
    pass P0
    {
		//SetRasterizerState(WireFrame);
		//SetRasterizerState(DisableCulling);   
        SetVertexShader(CompileShader(vs_4_0, VS()));
        SetGeometryShader(NULL);
        SetPixelShader(CompileShader(ps_4_0, PS()));
        SetBlendState(NoAlpha, float4( 0.0f, 0.0f, 0.0f, 0.0f ), 0xFFFFFFFF);
    }
}
